package ru.amse.kiselev.exec;


import ru.amse.kiselev.fsmeditor.automaton.Alphabet;
import ru.amse.kiselev.fsmeditor.automaton.Automaton;
import ru.amse.kiselev.fsmeditor.automaton.AutomatonFactory;
import ru.amse.kiselev.fsmeditor.automaton.Condition;
import ru.amse.kiselev.fsmeditor.automaton.IState;
import ru.amse.kiselev.fsmeditor.automaton.ISymbol;
import ru.amse.kiselev.fsmeditor.automaton.ITransition;
import ru.amse.kiselev.fsmeditor.automaton.Transition;
import ru.amse.kiselev.fsmeditor.interpreter.IInterpreter;
import ru.amse.kiselev.fsmeditor.interpreter.Interpreter;

public class TestDrawer {
	private Automaton<Alphabet> myAutomaton;
	private IState<Alphabet> stateA, stateB, stateC, stateD, stateE;
	private IInterpreter<Alphabet> myInterpreter;
	private ISymbol[] myCommands;
	private AutomatonFactory<Alphabet> myAutomatonFactory;
	
	public TestDrawer(AutomatonFactory<Alphabet> automatonFactory) {
		myAutomatonFactory = automatonFactory;
		makeAutomat();		
	}
	
	public IInterpreter<Alphabet> getInterpreter() {
		return myInterpreter;
	}
	
	@SuppressWarnings("unchecked")
	private void makeAutomat() {
		myAutomaton = new Automaton<Alphabet>();
		
		stateA = myAutomatonFactory.createState("A");
		stateB = myAutomatonFactory.createState("B");
		stateC = myAutomatonFactory.createState("C");
		stateD = myAutomatonFactory.createState("D");
		stateE = myAutomatonFactory.createState("E");
		
		ITransition<Alphabet> transition = new Transition<Alphabet>(stateA, stateB, new Condition<Alphabet>(Alphabet.INSTANCE.getSymbol("a")));
		myAutomaton.addTransition(transition);
		transition = new Transition<Alphabet>(stateA, stateC, new Condition<Alphabet>(Alphabet.INSTANCE.getSymbol("b")));
		myAutomaton.addTransition(transition);
		transition = new Transition<Alphabet>(stateB, stateD, new Condition<Alphabet>(Alphabet.INSTANCE.getSymbol("c")));
		myAutomaton.addTransition(transition);
		transition = new Transition<Alphabet>(stateB, stateE, new Condition<Alphabet>(Alphabet.INSTANCE.getSymbol("d")));
		myAutomaton.addTransition(transition);
		transition = new Transition<Alphabet>(stateE, stateC, new Condition<Alphabet>(Alphabet.INSTANCE.getSymbol("e")));
		myAutomaton.addTransition(transition);
		myAutomaton.setInitialState(stateA);
		stateC.setFinal();
		
		myCommands = new ISymbol[5];
		myCommands[0] = Alphabet.INSTANCE.getSymbol("a");
		myCommands[1] = Alphabet.INSTANCE.getSymbol("d");
		myCommands[2] = Alphabet.INSTANCE.getSymbol("e");
		myCommands[3] = Alphabet.INSTANCE.getSymbol("b");
		myCommands[4] = Alphabet.INSTANCE.getSymbol("c");
		
		myInterpreter = new Interpreter<Alphabet>(myAutomaton, myCommands);
		
	}
}
