package ru.amse.kiselev.fsmeditor.automaton;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

public class Automaton<A extends IAlphabet> implements IAutomaton<A> {
	private Set<IState<A>> myStates;
	private IState<A> myInitialState;

	public Automaton() {
		myStates = new HashSet<IState<A>>();
		myInitialState = null;
	}

	public void addState(IState<A> state) {
		myStates.add(state);
	}

	public void addTransition(ITransition<A> transition) {
		myStates.add(transition.getStart());
		myStates.add(transition.getEnd());

		transition.getStart().addTransition(transition);
	}

	public IState<A> getInitialState() {
		return myInitialState;
	}

	public Collection<IState<A>> getStates() {
		return Collections.unmodifiableCollection(myStates);
	}

	public void removeState(IState<A> state) {
		if (myInitialState == state) {
			myInitialState = null;
		}
		myStates.remove(state);
		for (IState<A> st : myStates) {
			Collection<ITransition<A>> tList = st.connectedWith(state);

			for (ITransition<A> tr : tList) {
				st.removeTransition(tr);
			}
		}
	}

	public void removeTransition(ITransition<A> transition) {
		transition.getStart().removeTransition(transition);
	}

	public void setInitialState(IState<A> s) {
		myInitialState = s;
	}

	public void removeAllStates() {
		myStates.clear();
		myInitialState = null;
	}

}
