package ru.amse.kiselev.fsmeditor.automaton;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class State<A extends IAlphabet> implements IState<A> {
	// contains array: symbol --> Transition
	private ITransition<A>[] myConnection;
	private String myLabel;
	private boolean myFinal;

	
	@SuppressWarnings("unchecked")
	public State (A alphabet) {
		myConnection = new Transition[alphabet.getSize()];
		myFinal = false;
	}

	public State (A alphabet, String label) {
		this(alphabet);
		myLabel = label;
	}

	public String getLabel() {
		return myLabel;
	}

	public ITransition<A> getTransition(ISymbol symbol) {
		return myConnection[symbol.getIndex()];
	}
	
	public IState<A> getNextState(ISymbol symbol) {
		if (myConnection[symbol.getIndex()] == null) {
			return null;			
		}
		return myConnection[symbol.getIndex()].getEnd();
	}

	public Collection<ITransition<A>> getTransitions() {
		ArrayList<ITransition<A>> connections = new ArrayList<ITransition<A>>();
		for (ITransition<A> tr : myConnection) {
			if (tr != null) {
				connections.add(tr);
			}
		}
		return connections;
	}

	public void setLabel(String label) {
		myLabel = label;
	}

	public void addTransition(ITransition<A> transition) {
		if (transition.getStart() != this) {
			throw new IllegalArgumentException("Wrong vertex");
		} else {
			myConnection[transition.getCondition().getSymbol().getIndex()] = transition;
		}
	}

	public boolean isFinal() {
		return myFinal;
	}

	public void setFinal() {
		myFinal = true;
	}

	public void setNonFinal() {
		myFinal = false;
	}

	public Collection<ITransition<A>> connectedWith(IState<A> state) {
		List<ITransition<A>> list = new ArrayList<ITransition<A>>();
		for (ITransition<A> tr : myConnection) {
			if (tr != null && tr.getEnd() == state) {
				list.add(tr);
			}
		}
		return list;
	}

	public void removeTransition(ITransition<A> transition) {
		myConnection[transition.getCondition().getSymbol().getIndex()] = null;
	}

}
