package ru.amse.kiselev.fsmeditor.graphics;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.stream.XMLStreamException;

import org.xml.sax.SAXException;

import ru.amse.kiselev.fsmeditor.automaton.AutomatonFactory;
import ru.amse.kiselev.fsmeditor.automaton.IAlphabet;
import ru.amse.kiselev.fsmeditor.graphics.representation.AutomatonLoader;
import ru.amse.kiselev.fsmeditor.graphics.representation.AutomatonSaver;
import ru.amse.kiselev.fsmeditor.graphics.representation.IGraphicalRepresentation;
import ru.amse.kiselev.fsmeditor.interpreter.IInterpreter;
import ru.amse.kiselev.fsmeditor.interpreter.Interpreter;

public class Drawer<A extends IAlphabet> {
	private JFrame myFrame = new JFrame("Automaton Modeller");
	private JPanel myImagePanel = new JPanel();
	private MouseHandler<A> myMouseHandler;
	// components
	private AutomatonComponent<A> myAutomatonComponent;
	private StringComponent<A> myStringComponent;
	// factory
	private AutomatonFactory<A> myAutomatonFactory;
	private AutomatonSaver<A> mySaver;
	private JFileChooser myFileChooser;
	private JMenuBar myMenuBar;
	private JScrollPane myScrollPane;

	public Drawer(IInterpreter<A> interpreter,
			AutomatonFactory<A> automatonFactory,
			AutomatonComponent<A> automatonComponent) {
		myAutomatonFactory = automatonFactory;
		myAutomatonComponent = automatonComponent;
		myMouseHandler = new MouseHandler<A>(myFrame, myImagePanel,
				myAutomatonFactory, myAutomatonComponent);
		myFrame.setSize(750, 750);
		myFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		//JPanel mainPanel = (JPanel) myFrame.getContentPane();
		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		
		addMenuComponents();
		
		JPanel stringPanel = addStringComponents(interpreter);
		stringPanel.setPreferredSize(new Dimension(700, 100));
		JPanel buttonsPanel = addButtonComponents();
		buttonsPanel.setPreferredSize(new Dimension(700, 60));
		
		addImageComponents(interpreter);
		myImagePanel.setPreferredSize(new Dimension(700, 400));
		
		mainPanel.add(buttonsPanel, BorderLayout.NORTH);
		mainPanel.add(stringPanel, BorderLayout.SOUTH);
		mainPanel.add(myImagePanel, BorderLayout.CENTER);
		
		myScrollPane = new JScrollPane(mainPanel);
		JPanel mainF = (JPanel) myFrame.getContentPane();
		mainF.setPreferredSize(new Dimension(700, 700));
		mainF.add(myScrollPane, BorderLayout.CENTER);
		
		mySaver = new AutomatonSaver<A>(myAutomatonComponent
				.getGraphicalRepresentation(), myFrame, myAutomatonFactory
				.getAlphabet());
	}

	public void init(IGraphicalRepresentation<A> graphicalRepresentation) {
		IInterpreter<A> interpreter = new Interpreter<A>(
				graphicalRepresentation.getAutomaton(), myAutomatonComponent
						.getInterpreter().getCommands());

		myAutomatonComponent.setData(interpreter, graphicalRepresentation);
		myImagePanel.removeMouseListener(myMouseHandler);
		myImagePanel.removeMouseMotionListener(myMouseHandler);
		myMouseHandler = new MouseHandler<A>(myFrame, myImagePanel,
				myAutomatonFactory, myAutomatonComponent);
		myImagePanel.addMouseMotionListener(myMouseHandler);
		myImagePanel.addMouseListener(myMouseHandler);
		mySaver = new AutomatonSaver<A>(graphicalRepresentation, myFrame,
				myAutomatonFactory.getAlphabet());
		myStringComponent.setInterpreter(interpreter);
	}

	public void show() {
		myFrame.setVisible(true);
	}

	private JPanel addStringComponents(IInterpreter<A> interpreter) {
		JPanel stringPanel = new JPanel();
		Border border = BorderFactory.createTitledBorder(new EtchedBorder()/*, "Command Line", */);
		stringPanel.setBorder(border);
		//stringPanel.setLayout(new GridLayout(1, 1));
		stringPanel.setLayout(new BoxLayout(stringPanel, BoxLayout.Y_AXIS));
		stringPanel.setPreferredSize(new Dimension(100, 100));
		
		myStringComponent = new StringComponent<A>(	50, 50, interpreter, myAutomatonFactory.getAlphabet(), myFrame);
		stringPanel.add(myStringComponent);
		
		
		//myStringComponent.setBorder(border);
		
		
		stringPanel.addMouseListener(myStringComponent.getStringMouse());
		stringPanel.addMouseMotionListener(myStringComponent.getStringMouse());
		
		return stringPanel;
	}

	private void addImageComponents(IInterpreter<A> interpreter) {
		myImagePanel.setLayout(new GridLayout(1, 1));

		myImagePanel.add(myAutomatonComponent);
		Border border = BorderFactory.createTitledBorder(new EtchedBorder()/*, "Automaton"*/);
		myImagePanel.setBorder(border);

		myImagePanel.addMouseMotionListener(myMouseHandler);
		myImagePanel.addMouseListener(myMouseHandler);
	}

	private JPanel addButtonComponents() {
		
		JToolBar toolbar = new JToolBar();
		
		JButton drawButton = new JButton("Step");
		drawButton.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent arg0) {
				int errorCode;
				if ((errorCode = myAutomatonComponent.getInterpreter().step()) < 0) {
					switch(errorCode) {
						case -1: JOptionPane.showMessageDialog(myFrame, "Final state reached!", "Over", JOptionPane.INFORMATION_MESSAGE); break;
						case -2: JOptionPane.showMessageDialog(myFrame, "Command line is over!", "Over", JOptionPane.INFORMATION_MESSAGE); break;
						case -3: JOptionPane.showMessageDialog(myFrame, "No suitable connection found!", "Over", JOptionPane.INFORMATION_MESSAGE); break;
						case -4: JOptionPane.showMessageDialog(myFrame, "Initaial state is not indicated", "Error", JOptionPane.INFORMATION_MESSAGE); break;
					}
					myAutomatonComponent.getInterpreter().reset();
				}
				myAutomatonComponent.repaint();
				myStringComponent.repaint();
			}
		});
		toolbar.add(drawButton);

		JButton resetButton = new JButton("Reset");
		resetButton.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = -349863674272341306L;
			public void actionPerformed(ActionEvent e) {
				myAutomatonComponent.getInterpreter().reset();
				myFrame.repaint();
			}
		});
		toolbar.add(resetButton);
		
		JButton vertexButton = new JButton("New Vertex");
		vertexButton.addActionListener(myMenuBar.getMenu(1).getItem(1).getActionListeners()[0]);
		toolbar.add(vertexButton);
		
		JButton transitionButton = new JButton("New Transition");
		transitionButton.addActionListener(myMenuBar.getMenu(1).getItem(0).getActionListeners()[0]);
		toolbar.add(transitionButton);
		
		JButton statusButton = new JButton("Final / Not Final");
		statusButton.addActionListener(myMenuBar.getMenu(1).getItem(3).getActionListeners()[0]);
		toolbar.add(statusButton);
		
		JButton initialButton = new JButton("Make Initial");
		initialButton.addActionListener(myMenuBar.getMenu(1).getItem(2).getActionListeners()[0]);
		toolbar.add(initialButton);
		
		JButton clearButton = new JButton("New Automaton");
		clearButton.addActionListener(myMenuBar.getMenu(0).getItem(0).getActionListeners()[0]);
		toolbar.add(clearButton);
		
		/*JButton clearStringButton = new JButton("New Command Line");
		clearStringButton.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = -349863674272341306L;
			public void actionPerformed(ActionEvent e) {
				myAutomatonComponent.getInterpreter().reset(new LinkedList<ISymbol>());
				myFrame.repaint();
			}
		});
		toolbar.add(clearStringButton);*/
		
		JPanel buttonsPanel = new JPanel();
		buttonsPanel.add(toolbar);
		buttonsPanel.setBorder(new EtchedBorder());
		
		Border buttonsBorder = BorderFactory.createTitledBorder(new EtchedBorder());
		buttonsPanel.setBorder(buttonsBorder);
		
		return buttonsPanel;
	}

	private void createFileChooser() {
		if (myFileChooser == null) {
			myFileChooser = new JFileChooser();
		}
	}
	private void addMenuComponents() {

		JMenuBar menuBar = new JMenuBar();
		JMenu menuFile = new JMenu("File");
		JMenu menuEdit = new JMenu("Edit");
		JMenu menuAbout = new JMenu("About");

		JMenuItem newOne = new JMenuItem("NewOne");
		newOne.setAction(new AbstractAction() {
			private static final long serialVersionUID = 4262825084009675322L;
			public void actionPerformed(ActionEvent e) {
				myAutomatonComponent.getGraphicalRepresentation().removeAllVertexes();
				init(myAutomatonComponent.getGraphicalRepresentation());
				myFrame.repaint();
			}
		});
		newOne.setText("New");
		menuFile.add(newOne);
		
		final JMenuItem loadFile = new JMenuItem("Load");
		loadFile.setAction(new AbstractAction() {
			private static final long serialVersionUID = -349863674272341306L;
			public void actionPerformed(ActionEvent e) {
				createFileChooser();
				int retval = myFileChooser.showOpenDialog(loadFile);
				if (retval == JFileChooser.APPROVE_OPTION) {
					File file = myFileChooser.getSelectedFile();
					try {
						Drawer.this.init((new AutomatonLoader<A>(
								myAutomatonFactory)).loadAutomaton(file));
						Drawer.this.myFrame.repaint();
					} catch (IOException e1) {
						JOptionPane.showMessageDialog(myFrame, "File opening error!", "Error",  JOptionPane.ERROR_MESSAGE);
					} catch (SAXException e1) {
						JOptionPane.showMessageDialog(myFrame, "XML File processing error!", "Error",  JOptionPane.ERROR_MESSAGE);
					} catch (ParserConfigurationException e2) {
						JOptionPane.showMessageDialog(myFrame, "XML configuration error!", "Error",  JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		});
		menuFile.add(loadFile);
		loadFile.setText("Load from file...");

		JMenuItem saveFile = new JMenuItem("Save");
		saveFile.setAction(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				createFileChooser();
				int retval = myFileChooser.showOpenDialog(loadFile);
				if (retval == JFileChooser.APPROVE_OPTION) {
					File file = myFileChooser.getSelectedFile();
					try {
						mySaver.saveAutomaton(file);
					} catch (XMLStreamException e1) {
						JOptionPane.showMessageDialog(myFrame, "XML converting error!", "Error",
								JOptionPane.ERROR_MESSAGE);
					} catch (IOException e2) {
						JOptionPane.showMessageDialog(myFrame, "File access error!", "Error",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		});
		menuFile.add(saveFile);
		saveFile.setText("Save to file...");

		JMenuItem exit = new JMenuItem("Exit");
		exit.setAction(new AbstractAction() {
			private static final long serialVersionUID = 4262825084009675322L;
			public void actionPerformed(ActionEvent e) {
				myFrame.dispose();
			}
		});
		exit.setText("Exit");
		menuFile.add(exit);
		
	
		
		JMenuItem about = new JMenuItem("About");
		about.setAction(new AbstractAction() {
			private static final long serialVersionUID = 0L;
			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(myFrame, "Automaton Modeller 1.0\nFinite Automaton Editor and Interpreter\n" + "author Yury Kiselev,\n2008",
						"About Automaton Modeller", JOptionPane.INFORMATION_MESSAGE);
			}
		});
		
		menuAbout.add(about);
		about.setText("About the program");

		JMenuItem addTransition = new JMenuItem("Add Transition");
		addTransition.setAction(new AbstractAction() {
			private static final long serialVersionUID = -349863674272341306L;

			public void actionPerformed(ActionEvent e) {
				myMouseHandler.setAddingTransitionMode();
			}
		});
		addTransition.setText("Add Transition");
		menuEdit.add(addTransition);

		JMenuItem addVertex = new JMenuItem("Add Vertex");
		addVertex.setAction(new AbstractAction() {
			private static final long serialVersionUID = -1547593175811530731L;

			public void actionPerformed(ActionEvent e) {
				myMouseHandler.setAddingVertexMode();
			}
		});
		
		addVertex.setText("Add Vertex");
		menuEdit.add(addVertex);

		JMenuItem setInitial = new JMenuItem("Set Initial");
		setInitial.setAction(new AbstractAction() {
			private static final long serialVersionUID = -1547593175811530731L;

			public void actionPerformed(ActionEvent e) {
				myMouseHandler.setInitialSetMode();
			}
		});
		setInitial.setText("Set Initial Vertex");
		menuEdit.add(setInitial);
		
		
		JMenuItem setFinal = new JMenuItem("Set Final");
		setFinal.setAction(new AbstractAction() {
			private static final long serialVersionUID = -492957418555353483L;

			public void actionPerformed(ActionEvent e) {
				myMouseHandler.setFinalSetMode();
			}
		});
		setFinal.setText("Change Final Status");
		menuEdit.add(setFinal);
		
		menuBar.add(menuFile);
		menuBar.add(menuEdit);
		menuBar.add(menuAbout);

		myFrame.setJMenuBar(menuBar);
		myMenuBar = menuBar;
	}

}
