package ru.amse.kiselev.fsmeditor.graphics;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.QuadCurve2D;

public class Drawing {
	public static final int TRANSITION_TOLERANCE = 15;
	public static final int BORDER_TOLERANCE = 5;
	public static final int CONDITION_INTERVAL = 5;
	public static final int CURVINESS = 20;
	public static final int POINT_RADIUS = 2;
	public static final int TRANSITION_RADIUS = 25;
	public static final int POINTER_X = 10;
	public static final int POINTER_Y = 4;
	
	public static final Font NAME_FONT= new Font("Verdana", Font.PLAIN, 15);
	
	
	public static AffineTransform drawStraightPointer
	(Graphics gc, Point start, Point end, Color color, int sRadius, int fRadius) {
		return drawQuadCurvePointer(gc, start, end, 0, color, sRadius, fRadius, 0);
	}
	public static AffineTransform drawStraightPointer
	(Graphics gc, double startX, double startY, 
			double endX, double endY, Color color, int sRadius, int fRadius) {
		return drawQuadCurvePointer(gc, startX, startY, endX, endY, 0, color, sRadius, fRadius, 0);
	}
	public static AffineTransform drawQuadCurvePointer
	(Graphics gc, Point start, Point end, int curviness, Color color, int sRadius, int fRadius, int direction) {
		return drawQuadCurvePointer(gc, start.getX(), start.getY(), end.getX(), end.getY(), curviness, color, sRadius, fRadius, direction);
	}
	
	
	public static AffineTransform drawQuadCurvePointer
	(Graphics gc, double startX, double startY, double endX, 
			double endY, int curviness, Color color, int sRadius, int fRadius, int direction) {
		Color pastColor = gc.getColor();
		gc.setColor(color);
		Graphics2D gc2 = (Graphics2D) gc;
		
		//bad, wrong assignment -- works for identical Radiuses only
		int centerX = (int) (startX + endX) / 2;
		int centerY = (int) (startY + endY) / 2;
		int lengthP = (int) Math.sqrt((startX - endX) * (startX - endX) + 
				(startY - endY) * (startY - endY));
		int length = lengthP - sRadius - fRadius;

		AffineTransform at = new AffineTransform();
		AffineTransform concatenated = gc2.getTransform();
		AffineTransform past = gc2.getTransform();

		
		at.translate(centerX, centerY);
		concatenated.translate(centerX, centerY);
		double angle = Math.atan2(endY - startY, endX - startX);
		if (angle > Math.PI / 2 || angle <= -Math.PI / 2) {
			at.translate(0, -CURVINESS * direction);	
		} else {
			at.translate(0, CURVINESS * direction);
		}
		at.rotate(angle);
		concatenated.rotate(angle);
		gc2.setTransform(concatenated);

		
		QuadCurve2D.Double quad = new QuadCurve2D.Double();

		Point2D.Double startQ = new Point2D.Double();
		Point2D.Double endQ = new Point2D.Double();
		Point2D.Double control = new Point2D.Double();

		startQ.setLocation(-length / 2, 0);
		endQ.setLocation(length / 2, 0);
		control.setLocation(0, curviness);

		quad.setCurve(startQ, control, endQ);
		
		gc2.draw(quad);
		
		double angleP = Math.atan2(curviness, length / 2);
		int pointerX = Math.min(length / 2, POINTER_X);
		int pointerY = Math.min(length / 5, POINTER_Y);
		drawPointer(gc2, angleP, length / 2.0, 0, pointerX, pointerY);
		
		AffineTransform resultedAT = at;
		if (angle > Math.PI / 2 || angle < -Math.PI / 2) {
			concatenated.rotate(Math.PI);
			at.rotate(Math.PI);
			resultedAT = at;
		}
		
		gc.setColor(pastColor);
		gc2.setTransform(past);
		return resultedAT;
	}
	
	public static void drawPointer(Graphics gc2, double angle, double startX, double startY, double pointerX, double pointerY) {
		gc2.drawLine((int) startX, (int) startY, (int) (startX - pointerX * Math.cos(angle) + Math.sin(angle) * pointerY), (int) (startY + pointerX * Math.sin(angle) + Math.cos(angle) * pointerY));
		gc2.drawLine((int) startX, (int) startY, (int) (startX - pointerX * Math.cos(angle) - Math.sin(angle) * pointerY), (int) (startY + pointerX * Math.sin(angle) - Math.cos(angle) * pointerY));
	}
	
	public static void drawPointer(Graphics gc2, double angle, double startX, double startY) {
		drawPointer(gc2, angle, startX, startY, POINTER_X, POINTER_Y);
	}
	
}
