package ru.amse.kiselev.fsmeditor.graphics;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import ru.amse.kiselev.fsmeditor.automaton.Alphabet;
import ru.amse.kiselev.fsmeditor.automaton.IAlphabet;
import ru.amse.kiselev.fsmeditor.automaton.ISymbol;
import ru.amse.kiselev.fsmeditor.interpreter.IInterpreter;

public class StringComponent<A extends IAlphabet> extends JComponent {
	public class MouseClickListener extends MouseAdapter {
		public void mouseClicked(MouseEvent e) {
			if (myStringBounds.contains(e.getPoint())) {
				List<ISymbol> newString = editCommands();
				if (newString != null) {
					myInterpreter.reset(newString);
					myFrame.repaint();
				}
			}
		}

		private final List<ISymbol> editCommands() {
			String s = (String) JOptionPane.showInputDialog(myFrame,
					"Enter new commands: \n", "Use space as a delimeter",
					JOptionPane.PLAIN_MESSAGE, null, null, getString());
			if (s == null) {
				return null;
			}

			if (s.length() == 0) {
				JOptionPane.showMessageDialog(myFrame,
						"Please enter non empty command string.", "Try again",
						JOptionPane.ERROR_MESSAGE);
				return null;
			} else {
				List<ISymbol> output = new ArrayList<ISymbol>();
				StringTokenizer st = new StringTokenizer(s);
				ISymbol symbol;
				for (; st.hasMoreTokens();) {
					String token = st.nextToken();
					if (token.length() != 0) {
						symbol = myAlphabet.getSymbol(token);
						if (symbol != null) {
							output.add(symbol);
						} else {
							JOptionPane.showMessageDialog(myFrame,
									"Wrong commands", "Try again",
									JOptionPane.ERROR_MESSAGE);
							return null;
						}
					}
				}
				if (output.size() > 20) {
					JOptionPane.showMessageDialog(myFrame,
							"Please enter no more than 20 commands.",
							"Try again", JOptionPane.ERROR_MESSAGE);
					return null;
				}
				return output;
			}
		}

		private String getString() {
			StringBuffer sb = new StringBuffer();
			for (ISymbol symbol : myInterpreter.getCommands()) {
				sb.append(myAlphabet.getString(symbol));
				sb.append(" ");
			}
			if (sb.length() > 0) {
				sb.deleteCharAt(sb.length() - 1);
			}
			return sb.toString();
		}
	}

	private static final long serialVersionUID = 1L;
	private static final int BORDER = 3;
	private static final int MORE = 20;
	private static final int LENGTH = 600;
	private int myX;
	private int myY;
	private double myScaling = 2.0;
	private IInterpreter<A> myInterpreter;
	private Shape myStringBounds;
	private A myAlphabet;
	private JFrame myFrame;
	private MouseAdapter myMouseListener;

	private Font myCommandsFont = new Font("Georgia", Font.BOLD, 20);
	

	public StringComponent(int x, int y,
			IInterpreter<A> interpreter, A alphabet, JFrame frame) {
		myInterpreter = interpreter;
		myX = x;
		myY = y;
		myAlphabet = alphabet;
		myFrame = frame;
		myMouseListener = new MouseClickListener();
	}

	public void setX(int x) {
		myX = x;
	}

	public void setY(int y) {
		myY = y;
	}

	public void paint(Graphics gc) {
		((Graphics2D) gc).setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		gc.setFont(myCommandsFont);

		Graphics2D gc2 = (Graphics2D) gc;

		int i = 0;
		int current = 0;
		int offset = myInterpreter.getCurrentOffset();
		FontMetrics metrics = gc2.getFontMetrics();
		int firstPixel = myX;
		Collection<ISymbol> symbolString = myInterpreter.getCommands();
		for (ISymbol symbol : symbolString) {
			String toPaint = Alphabet.INSTANCE.getString(symbol.getIndex());
			int width = metrics.stringWidth(toPaint + "");
			if (i < offset) {
				gc2.setColor(Color.blue);
				gc2.fillRect(firstPixel + current - BORDER, myY - metrics.getHeight() - BORDER, 
						(int) (width * myScaling), metrics.getHeight() + 2 * BORDER);
				gc2.setColor(Color.red);
			} 
			gc2.setColor(Color.red);
			
			gc2.drawString(toPaint + "", current + myX, myY);
			current += width * myScaling;
			i++;
		}
		gc2.setColor(Color.BLUE);
		Drawing.drawPointer(gc2, 0, firstPixel - BORDER - MORE, myY
				- metrics.getHeight() / 2, 15, 10);
		Drawing.drawPointer(gc2, 0, firstPixel - BORDER - MORE + 8, myY
				- metrics.getHeight() / 2, 15, 10);
		gc2.setColor(Color.BLACK);
		Rectangle2D bounds = new Rectangle2D.Double(firstPixel - BORDER, myY
				- metrics.getHeight() - BORDER, 
				LENGTH, metrics.getHeight() + 2 * BORDER);
		myStringBounds = bounds;
		gc2.draw(bounds);
		gc2.setFont(Drawing.NAME_FONT);
		gc2.drawString("Command Line", 15, 15);
	}

	public void setInterpreter(IInterpreter<A> interpreter) {
		myInterpreter = interpreter;
		myInterpreter.reset();
	}
	
	public MouseAdapter getStringMouse() {
		return myMouseListener;
	}
}
