package ru.amse.kiselev.fsmeditor.graphics.representation;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import ru.amse.kiselev.fsmeditor.automaton.Automaton;
import ru.amse.kiselev.fsmeditor.automaton.AutomatonFactory;
import ru.amse.kiselev.fsmeditor.automaton.Condition;
import ru.amse.kiselev.fsmeditor.automaton.IAlphabet;
import ru.amse.kiselev.fsmeditor.automaton.IAutomaton;
import ru.amse.kiselev.fsmeditor.automaton.IState;
import ru.amse.kiselev.fsmeditor.automaton.ISymbol;

public class AutomatonLoader<A extends IAlphabet> {
	private final class InputHandler extends DefaultHandler {
		private boolean myAutomatonReading;

		private AutomatonFactory<A> myAutomatonFactory;

		private int myInitialState;
		private ArrayList<IState<A>> myStates = new ArrayList<IState<A>>();
		private ArrayList<EmptyTransition> myTransitions = new ArrayList<EmptyTransition>();
		private int myCurrentVertex;

		public InputHandler(AutomatonFactory<A> automatonFactory, A alphabet) {
			myAutomatonFactory = automatonFactory;
		}

		public void startElement(String uri, String raw_name,
				String local_name, Attributes amap) throws SAXException {

			if (local_name.equals(SAXUtils.automatonS)) {
				myAutomatonReading = true;
				String s = amap.getValue(SAXUtils.initialStateS);
				if (!s.equals(SAXUtils.nullS)) {
					myInitialState = Integer.parseInt(s);
				} else {
					myInitialState = -1;
				}
			}

			if (myAutomatonReading) {
				if (local_name.equals(SAXUtils.automatonS)) {
					return;
				}
				if (local_name.equals(SAXUtils.stateS)) {
					String label = format(amap.getValue(SAXUtils.labelS));
					IState<A> state = myAutomatonFactory.createState(label);
					if (amap.getValue(SAXUtils.finalS).equals(SAXUtils.trueS)) {
						state.setFinal();
					}
					myStates.add(state);
				} else if (local_name.equals(SAXUtils.transitionS)) {
					String condition = format(amap.getValue(SAXUtils.conditionS));
					myTransitions.add(new EmptyTransition(new Integer(amap
							.getValue(SAXUtils.fromS)), new Integer(amap
							.getValue(SAXUtils.toS)), condition));
				} else {
					throw new IllegalArgumentException(local_name);
				}
			} else {
				if (local_name.equals(SAXUtils.vertexS)) {
					int x = Integer.parseInt(amap.getValue(SAXUtils.xS));
					int y = Integer.parseInt(amap.getValue(SAXUtils.yS));

					IVertex<A> currentVertex = myGRP.getVertex(myStates
							.get(myCurrentVertex));
					currentVertex.setLocation(x, y);
				} else if (!local_name.equals(SAXUtils.grpS)) {
					throw new IllegalArgumentException(local_name);
				}
				myCurrentVertex++;
			}
		}

		private String format(String value) {
			if (value.length() < 2 || (value.charAt(0) != '#')) {
				return value;
			}
			for (int i = 1; i < value.length(); i++) {
				char c = value.charAt(i);
				if (c < '0' || c > '9') {
					return value;
				}
			}
			StringBuffer sb = new StringBuffer();
			for (int i = 1; i < value.length(); i++) {
				sb.append(value.charAt(i));
			}
			Integer charNumber = Integer.parseInt(sb.toString());
			return ((char) charNumber.intValue()) + "";
		}

		public void endElement(String uri, String raw_name, String local_name)
				throws SAXException {
			if (local_name.equals(SAXUtils.automatonS)) {
				myAutomatonReading = false;

				for (int i = 0; i < myStates.size(); i++) {
					myAutomaton.addState(myStates.get(i));
				}

				for (int i = 0; i < myTransitions.size(); i++) {
					ISymbol symbol = myAutomatonFactory.getAlphabet()
							.getSymbol(myTransitions.get(i).getCondition());

					myAutomaton.addTransition(myAutomatonFactory
							.createTransition(myStates.get(myTransitions.get(i)
									.getFirst()), myStates.get(myTransitions
									.get(i).getSecond()), new Condition<A>(
									symbol)));
				}
				if (myInitialState != -1) {
					myAutomaton.setInitialState(myStates.get(myInitialState));
				}

				myGRP = new GraphicalRepresentation<A>(myAutomaton);
				myCurrentVertex = 0;
			}
		}

		public void startDocument() throws SAXException {
		}

		public void endDocument() throws SAXException {
		}

	}

	private AutomatonFactory<A> myFactory;
	private IAutomaton<A> myAutomaton;
	private IGraphicalRepresentation<A> myGRP;

	public AutomatonLoader(AutomatonFactory<A> aFactory) {
		myFactory = aFactory;
		myAutomaton = new Automaton<A>();
		myGRP = new GraphicalRepresentation<A>(myAutomaton);
	}

	public IGraphicalRepresentation<A> loadAutomaton(File file)
			throws IOException, SAXException, ParserConfigurationException {
		javax.xml.parsers.SAXParser reader = SAXParserFactory.newInstance()
				.newSAXParser();
		InputHandler handler = new InputHandler(myFactory, myFactory
				.getAlphabet());
		reader.parse(file, handler);
		return myGRP;
	}
}
