package ru.amse.kiselev.fsmeditor.graphics.representation;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;

import javax.swing.JFrame;
import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import ru.amse.kiselev.fsmeditor.automaton.IAlphabet;
import ru.amse.kiselev.fsmeditor.automaton.IAutomaton;
import ru.amse.kiselev.fsmeditor.automaton.IState;
import ru.amse.kiselev.fsmeditor.automaton.ITransition;

public class AutomatonSaver<A extends IAlphabet> {
	private IGraphicalRepresentation<A> myGRP;
	private A myAlphabet;
	XMLStreamWriter myWriter;
	FileOutputStream myStream;
	IAutomaton<A> myAutomaton;
	Collection<IVertex<A>> myVertexes;
	HashMap<IState<A>, Integer> myNums;
	
	public AutomatonSaver(IGraphicalRepresentation<A> grp, JFrame frame,
			A alphabet) {
		myGRP = grp;
		myAlphabet = alphabet;
	}

	public void saveAutomaton(File file) throws IOException, XMLStreamException {
		try {
			myStream = new FileOutputStream(file);
		} catch (FileNotFoundException e) {
			throw e;
		}

		try {
			myWriter = XMLOutputFactory.newInstance().createXMLStreamWriter(
					myStream);
		} catch (XMLStreamException e) {
			try {
				myStream.close();
			} catch (IOException e1) {
				throw e;
			}
			return;
		} catch (FactoryConfigurationError e) {
			try {
				myStream.close();
			} catch (IOException e1) {
				throw e;
			}
			return;
		}

		myAutomaton = myGRP.getAutomaton();
		myVertexes = myGRP.getVertexes();
		myNums = new HashMap<IState<A>, Integer>();
		
		int i = 0;
		for (IVertex<A> vertex : myVertexes) {
			myNums.put(vertex.getState(), i);
			i++;
		}
		try {
			myWriter.writeStartDocument();
			writeLeadingElements();
			writeStates();
			writeTransitions();
			
			myWriter.writeEndElement();
			
			writeVertexes();
			
			myWriter.writeEndElement();
			myWriter.writeEndDocument();
			myWriter.flush();
			myWriter.close();
		} catch (XMLStreamException e) {
			throw e;
		} finally {
			try {
				myStream.close();
			} catch (IOException e1) {
				throw e1;
			}
		}
		
	}

	private void writeAttribute(String name, String value) throws XMLStreamException, IOException {
		myWriter.flush();
		myStream.write((" " + name + "=\"" + value + "\"").getBytes());
		myStream.flush();
	}

	private void writeLeadingElements() throws XMLStreamException {
		myWriter.writeStartElement(SAXUtils.grpS);
		myWriter.writeStartElement(SAXUtils.automatonS);
		myWriter.writeAttribute(SAXUtils.initialStateS, "" + myNums.get(myAutomaton.getInitialState()));
	}
	
	private void writeTransitions() throws XMLStreamException, IOException {
		for (IVertex<A> vertex : myVertexes) {
			Collection<ITransition<A>> transitions = vertex.getState()
					.getTransitions();
			for (ITransition<A> transition : transitions) {
				myWriter.writeStartElement(SAXUtils.transitionS);
				myWriter.writeAttribute(SAXUtils.fromS, ""
						+ myNums.get(transition.getStart()));
				myWriter.writeAttribute(SAXUtils.toS, ""
						+ myNums.get(transition.getEnd()));
				String cond = myAlphabet.getString(transition
						.getCondition().getSymbol());
				if (ok(cond)) {
					myWriter.writeAttribute(SAXUtils.conditionS, cond);
				} else {
					writeAttribute(SAXUtils.conditionS, format(cond));
				}
				myWriter.writeEndElement();
			}
		}
	}
	
	private String format(String cond) {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < cond.length(); i++) {
			if (cond.charAt(i) > 127) {
				sb.append('&');
				sb.append('#');
				sb.append(((int) cond.charAt(i)) + "");
				sb.append(';');
			} else if (cond.charAt(i) < '!') {
				sb.append('#');
				sb.append(((int) cond.charAt(i)) + "");
			}else {
				sb.append(cond.charAt(i));
			}
		}
		return sb.toString();
	}

	private void writeStates() throws XMLStreamException, IOException{
		for (IVertex<A> vertex : myVertexes) {
			myWriter.writeStartElement(SAXUtils.stateS);
			myWriter.writeAttribute(SAXUtils.finalS, "" + vertex.getState().isFinal());
			if (ok(vertex.getState().getLabel())) {
				myWriter.writeAttribute(SAXUtils.labelS, vertex.getState().getLabel());
			} else {
				writeAttribute(SAXUtils.labelS, format(vertex.getState().getLabel()));
			}			
			myWriter.writeEndElement();
		}
	}
	
	private boolean ok(String label) {
		for (int i = 0; i < label.length(); i++) {
			if (label.charAt(i) < '!') {
				return false;
			}
		}
		return true;
	}

	private void writeVertexes() throws XMLStreamException{
		for (IVertex<A> vertex : myVertexes) {
			myWriter.writeStartElement(SAXUtils.vertexS);
			myWriter.writeAttribute(SAXUtils.xS, "" + vertex.getX());
			myWriter.writeAttribute(SAXUtils.yS, "" + vertex.getY());
			myWriter.writeEndElement();
		}
	}
}
