package ru.amse.kiselev.fsmeditor.graphics.representation;

import java.awt.geom.Point2D;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import ru.amse.kiselev.fsmeditor.automaton.IAlphabet;
import ru.amse.kiselev.fsmeditor.automaton.IAutomaton;
import ru.amse.kiselev.fsmeditor.automaton.IState;

/**
 * contains All data needed for the reloading the Automat and it's Graphical
 * representation from disk
 * 
 * @author Yury Kiselev
 * 
 * @param <A>
 */
public class GraphicalRepresentation<A extends IAlphabet> implements
		IGraphicalRepresentation<A> {
	private Map<IState<A>, IVertex<A>> myVertexMap;
	private IAutomaton<A> myAutomaton;

	public GraphicalRepresentation(IAutomaton<A> automaton) {
		myVertexMap = new HashMap<IState<A>, IVertex<A>>();
		myAutomaton = automaton;

		addAll(automaton);
	}

	public boolean addVertex(IVertex<A> v) {
		if (myVertexMap.containsKey(v.getState())) {
			return false;
		}

		myVertexMap.put(v.getState(), v);
		myAutomaton.addState(v.getState());
		return true;
	}

	// true if it's removed an item
	public boolean removeVertex(IVertex<A> v) {
		if (myVertexMap.containsKey(v.getState())) {
			myVertexMap.remove(v.getState());
			myAutomaton.removeState(v.getState());
			return true;
		} else {
			return false;
		}
	}

	public final void addAll(IAutomaton<A> automaton) {
		myVertexMap.clear();

		Collection<IState<A>> collection = automaton.getStates();

		int i = 0;
		int R = 100;
		int x0 = 150;
		int y0 = 150;
		int amount = collection.size();

		for (IState<A> state : collection) {
			i++;
			Point2D point = new Point2D.Double(x0 + R
					* Math.cos(2 * i * Math.PI / amount), (y0 + R
					* Math.sin(2 * i * Math.PI / amount)));
			Vertex<A> vertex = new Vertex<A>(point, state);
			myVertexMap.put(state, vertex);
		}

	}

	public Collection<IVertex<A>> getVertexes() {
		return Collections.unmodifiableCollection(myVertexMap.values());
	}

	public IVertex<A> getVertex(IState<A> state) {
		return myVertexMap.get(state);
	}

	public IAutomaton<A> getAutomaton() {
		return myAutomaton;
	}

	public void removeAllVertexes() {
		myVertexMap.clear();
		myAutomaton.removeAllStates();
	}

}
