package ru.amse.kiselev.fsmeditor.graphics.representation;

import java.awt.geom.Point2D;

import ru.amse.kiselev.fsmeditor.automaton.IAlphabet;
import ru.amse.kiselev.fsmeditor.automaton.IState;

public class Vertex<A extends IAlphabet> implements IVertex<A>{
	private Point2D myPoint;
	private int myRadius;
	private IState<A> myState;
	
	public Vertex(Point2D p, int r, IState<A> state) {
		myPoint = (Point2D) p.clone();
		myRadius = r;		
		myState = state;
	}
	
	public Vertex(Point2D p, IState<A> state) {
		this(p, IVertex.DEFAULT_RADIUS, state);	
	}
	
	public Vertex() {
		this(null, 0, null);		
	}
	
	//too many instances will be created in case of using this method
	/*public Point2D getPoint() {
		return myPoint.clone();		
	}*/
	public int getX() {
		return (int) myPoint.getX();
	}
	
	public int getY() {
		return (int) myPoint.getY();
	}

	public IState<A> getState() {
		return myState;		
	}
	
	public void setLocation(Point2D p) {
		//myPoint = (Point2D) p.clone();
		myPoint.setLocation(p);
	}
	
	public void setLocation(int x, int y) {
		//myPoint = new Point(x, y);
		myPoint.setLocation(x, y);
	}
	
	public int getRadius() {
		return myRadius;		
	}

	public double distance(Point2D p) {
		return myPoint.distance(p);
	}

	public double distance(int x, int y) {
		return Math.sqrt((x - myPoint.getX()) * (x - myPoint.getX()) + (y - myPoint.getY()) * (y - myPoint.getY()));
	}
	
	public String getString() {
		return myState.getLabel();
	}
}
