package ru.amse.kiselev.fsmeditor.interpreter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import ru.amse.kiselev.fsmeditor.automaton.IAlphabet;
import ru.amse.kiselev.fsmeditor.automaton.IAutomaton;
import ru.amse.kiselev.fsmeditor.automaton.IState;
import ru.amse.kiselev.fsmeditor.automaton.ISymbol;

public class Interpreter<A extends IAlphabet> implements IInterpreter<A> {
	private int myCurrentOffset;
	private IState<A> myCurrentState;
	private IAutomaton<A> myAutomaton;
	private List<ISymbol> myString;

	public Interpreter(IAutomaton<A> automaton, ISymbol commands[]) {
		myAutomaton = automaton;
		myString = new ArrayList<ISymbol>();
		for (int i = 0; i < commands.length; i++) {
			myString.add(commands[i]);	
		}
		reset();
	}
	
	public Interpreter(IAutomaton<A> automaton, Collection<ISymbol> commands) {
		myAutomaton = automaton;
		myString = new ArrayList<ISymbol>(commands);
		reset();
	}

	public Collection<ISymbol> getCommands() {
		return Collections.unmodifiableList(myString);
	}
 	
	public IAutomaton<A> getAutomaton() {
		return myAutomaton;
	}

	public int getCurrentOffset() {
		return myCurrentOffset;
	}

	public IState<A> getCurrentState() {
		return myCurrentState;
	}

	public boolean isAccepted() {
		return myCurrentState.isFinal();
	}

	public final void reset() {
		myCurrentOffset = -1;
		myCurrentState = null;
	}

	public int step() {
		if (myCurrentState != null && myCurrentState.isFinal()) {
			return -1;
		}
		
		if (myCurrentOffset >= myString.size()) {
			myCurrentOffset = -1;
			return -2;
		}
		
		if (myCurrentOffset == -1 && myCurrentState == null) {
			myCurrentOffset = 0;
			myCurrentState = myAutomaton.getInitialState();
			if (myCurrentState == null) {
				return -4;
			}
		} else {
			myCurrentState = myCurrentState.getNextState(myString.get(myCurrentOffset));
			myCurrentOffset++;			
		}
		if (myCurrentState != null) {
			return 1;
		} else {
			return -3;			
		}
	}

	public void reset(List<ISymbol> newString) {
		myCurrentOffset = -1;
		myCurrentState = null;
		myString = newString;
	}

}
